# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# Copyright 2014 Canonical
# Author: Omer Akram <omer.akram@canonical.com>
#
# This file is part of ubuntu-integration-tests.
# dialer-app is free software: you can redistribute it and/or modify it
# under the terms of the GNU General Public License version 3, as published
# by the Free Software Foundation.

from autopilot.matchers import Eventually
from autopilot.platform import model
from testtools.matchers import Equals
from testtools import skipIf, skipUnless

import subprocess
import sys
import time

from ubuntu_integration_tests.tests import IntegrationTestCase


# ensure we have an ofono account; we assume that we have these tools,
# otherwise we consider this a test failure (missing dependencies)
def tp_has_ofono():
    mc_tool = subprocess.Popen(['mc-tool', 'list'], stdout=subprocess.PIPE,
                               universal_newlines=True)
    mc_accounts = mc_tool.communicate()[0]
    return 'ofono/ofono/account' in mc_accounts

if not tp_has_ofono():
    subprocess.check_call(['ofono-setup'])
    if not tp_has_ofono():
        sys.stderr.write('ofono-setup failed to create ofono account!\n')
        sys.exit(1)

# determine whether we are running with phonesim
try:
    out = subprocess.check_output(["/usr/share/ofono/scripts/list-modems"],
                                  stderr=subprocess.PIPE)
    have_phonesim = out.startswith("[ /phonesim ]")
except subprocess.CalledProcessError:
    have_phonesim = False


@skipUnless(have_phonesim,
            "this test needs to run under with-ofono-phonesim")
@skipIf(model() == 'Desktop',
        "Needs to run on platform running unity8")
class TestSnapDecisionInegration(IntegrationTestCase):

    def setUp(self):
        super(TestSnapDecisionInegration, self).setUp()
        self._invoke_incoming_call()
        self._wait_for_incoming_call()

        self.addCleanup(
            subprocess.call, ['/usr/share/ofono/scripts/hangup-all']
        )

    def _get_accept_button(self, unity):
        return unity.select_single(objectName='button0', visible=True)

    def _get_decline_button(self, unity):
        return unity.select_single(objectName='button1', visible=True)

    def _wait_for_incoming_call(self):
        """Wait up to 5 s for an incoming phone call"""

        timeout = 10
        while timeout >= 0:
            out = subprocess.check_output(
                ["/usr/share/ofono/scripts/list-calls"],
                stderr=subprocess.PIPE)
            if "State = incoming" in out:
                break
            timeout -= 1
            time.sleep(0.5)
        else:
            self.fail("timed out waiting for incoming phonesim call")

    def _invoke_incoming_call(self):
        # magic number 199 will cause a callback from 1234567; dialing 199
        # itself will fail, so quiesce the error
        subprocess.call(["/usr/share/ofono/scripts/dial-number", "199"],
                        stdout=subprocess.PIPE, stderr=subprocess.PIPE)

    def _accept_call_from_ui(self):
        unity = self.helpers.get_proxy_object_by_process_name('unity8')
        accept_button = self._get_accept_button(unity)

        self.pointing_device.click_object(accept_button)

    def test_unity_accept_button_open_dialer(self):
        """Ensure tapping on 'accept' button in the OSD notification
        opens up the dialer-app in live call.

        """
        self._accept_call_from_ui()

        dialer = self.helpers.get_proxy_object_by_process_name('dialer-app')
        dialer_main_view = dialer.wait_select_single("QQuickView")
        live_call = dialer.wait_select_single(objectName='pageLiveCall')

        self.assertThat(dialer_main_view.visible, Eventually(Equals(True)))
        self.assertThat(live_call.visible, Eventually(Equals(True)))

        self.addCleanup(subprocess.call, ['pkill', 'dialer-app'])
